﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TechNote70_TCPServer_CS
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // 프레임 시작 구분코드
        private const string STXCODE = "@";
        // 프레임 종료 구분코드
        private const string ETXCODE = ";";

        // 연결된 클라이언트의 IP 주소
        private string m_strClientIPAddress;
        // 총 수신데이터 카운트
        private int recvTotalCnt = 0;
        // 총 수신에러 카운트
        private int recvErrorCnt = 0;
        // 송신 카운트
        private int m_iSendCnt = 0;
        // CRC32데이터
        private string m_CRC32Data = "";
        // 전송데이터
        private string m_sSendData = "";
        private string m_strReceive;

        public enum VERIFYCODE
        {
            SUCESS = 0,         // 성공
            LENGTH_ERROR = 1,   // 길이에러
            CRC_ERROR = 2,      // CRC에러
            ERROR = 3,          // 그 이외 에러
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // 현재 Form을 MainForm으로 설정합니다.
            smartForm1.MainForm = this;
            // 전송할 데이터의 길이를 출력
            labDataLength.Text = labSendData.Text.Length.ToString();
        }

        // 서버 시작
        private void btnServerStart_Click(object sender, EventArgs e)
        {
            // SmartFile의 경로를 설정하고 파일 Open
            smartFile1.FilePathName = "Flash Disk\\Test1.txt";
            smartFile1.Open();

            // SmartTCPMultiServer 초기화
            smartTCPMultiServer1.Port = 3377;
            smartTCPMultiServer1.MaxClient = 30;
            smartTCPMultiServer1.MaxReceiveBufferSize = 500000;
            smartTCPMultiServer1.ReceiveTimeout = 3000;
            // 송신 데이터
            m_sSendData = labSendData.Text;

            if (smartTCPMultiServer1.IsStart == false)
            {
                smartTCPMultiServer1.Start();
            }

            if (smartTCPMultiServer1.IsStart == true)
            {
                labServerStatus.Text = "시작";
            }
            else
            {
                labServerStatus.Text = "시작X";
            }
        }

        // 파일 쓰기(일괄처리)
        private void btnFileWrite_Click(object sender, EventArgs e)
        {
            smartFile1.StringType.WriteBufferSave();
            smartFile1.Close();
        }

        // 프로그램 종료
        private void btnServerExit_Click(object sender, EventArgs e)
        {
            if (smartTCPMultiServer1.IsStart == true)
            {
                smartTCPMultiServer1.Stop();
            }
            smartForm1.Close();
        }

        // 리스트 박스 클리어
        private void btnClear_Click(object sender, EventArgs e)
        {
            lstRecvList.ClearAll();
        }

        // 데이터 전송
        private void btnSend_Click(object sender, EventArgs e)
        {
            smartTimer1.Interval = Convert.ToInt32(udInterval.Value);
            m_iSendCnt = 0;
            smartTimer1.Start();
        }

        private void smartTimer1_Tick(object sender, EventArgs e)
        {
            if (udSendCount.Value < ++m_iSendCnt)
            {
                smartTimer1.Stop();
                m_iSendCnt = 0;
            }
            else
            {
                byte[] StrByte = Encoding.Unicode.GetBytes(m_sSendData);
                m_CRC32Data = SmartX.SmartTCPMultiServer.GetCRC32Gen(StrByte).ToString("X");

                // Frame Format : STXCODE(1) + LENGTH(5) + CRC(8) + DATA(N) + ETXCODE(1)
                // 모든 문자열은 Unicode 문자열로 처리됨
                smartTCPMultiServer1.SendStringUnicodeIP(m_strClientIPAddress, STXCODE + m_sSendData.Length.ToString("00000") + m_CRC32Data + m_sSendData + ETXCODE);
                labSendCount.Text = (m_iSendCnt).ToString();
            }
        }

        // 수신 이벤트
        private void smartTCPMultiServer1_OnReceiveHandler(SmartX.SmartTCPMultiServer.CHandleClient.READINFO datas)
        {
            // 수신된 데이터를 Unicode형식으로 변경
            string strRecData;
            // GetFrameData에서 반환받은 프레임구조의 데이터 저장
            string strFrameData;

            strRecData = SmartX.SmartTCPMultiServer.ConvertUnicodeByteToString(datas.receiveDatas);

            // 프레임 구조가 될때까지 m_strReceive에 데이터를 누적
            m_strReceive += strRecData;

            if (strRecData != "")
            {
                // m_strReceive의 데이터가 공백이 될때까지 strFrameData(STXCODE(1) + LENGTH(5) + CRC(8) + DATA(N) + ETXCODE(1))구조로 완성하여 출력
                do
                {
                    // GetFrameData를 호출
                    strFrameData = GetFrameData(ref m_strReceive);
                    if (strFrameData != "")
                    {
                        switch (Verification(strFrameData))
                        {
                            // 정상수신
                            case VERIFYCODE.SUCESS:
                                labSuccessCnt.Text = (++recvTotalCnt).ToString();
                                lstRecvList.AddItem("[" + recvTotalCnt.ToString() + "] " + strFrameData);

                                smartFile1.StringType.WriteBuffer(strFrameData + "\r\n\r\n");
                                break;

                            // CRC 오류발생
                            case VERIFYCODE.CRC_ERROR:
                                labErrorCnt.Text = (++recvErrorCnt).ToString();
                                labErrorMsg.Text = "CRC_ERROR";
                                break;
                            // 수신 데이터 길이오류발생
                            case VERIFYCODE.LENGTH_ERROR:
                                labErrorCnt.Text = (++recvErrorCnt).ToString();
                                labErrorMsg.Text = "LENGTH_ERROR";
                                break;
                            // 그외 오류발생
                            case VERIFYCODE.ERROR:
                                labErrorCnt.Text = (++recvErrorCnt).ToString();
                                labErrorMsg.Text = "ERROR";
                                break;
                        }
                    }
                }
                while (strFrameData != "");
            }
        }

        // 프레임(STX+LENGTH+CRC32+데이터+ETX)구조로 만들어질 때까지 ""을 리턴하고 
        // 프레임 구조로 만들어진 경우 해당 프레임 부분을 리턴하고 프레임을 제외한 나머지 부분은 strInData에 저장
        private string GetFrameData(ref string strInData)
        {
            // 프레임에서 STX의 인덱스
            int iPosSTX;
            // 프레임에서 ETX의 인덱스
            int iPosETX;
            // 수신데이터
            string strTemp;
            // 리턴데이터(완전한 프레임)
            string strRet;

            strTemp = strInData;

            // strTemp에서 맨 처음 발견되는 지정된 유니코드 문자('@')의 0부터 시작하는 인덱스를 iPosSTX에저장
            iPosSTX = strTemp.IndexOf(STXCODE);

            // strTemp에서'@'를 발견하지 못한경우
            if (iPosSTX == -1)
            {
                return "";
            }

            // strTemp에서 맨 처음 발견되는 지정된 유니코드 문자(';')의 0부터 시작하는 인덱스를 iPosETX에 저장
            iPosETX = strTemp.IndexOf(ETXCODE);

            // strTemp에서';'를 발견하지 못한경우
            if (iPosETX == -1)
            {
                return "";
            }

            // @와;가 모두 있어야만 이부분 실행
            // iPosSTX 부터 (iPosETX - iPosSTX) + 1까지의 문자를 strRet에 저장
            strRet = strTemp.Substring(iPosSTX, (iPosETX - iPosSTX) + 1);

            // iPosSTX 부터 (iPosETX - iPosSTX) + 1까지의 문자를 삭제
            // strTemp - strRet나머지를 strInData에저장. GetFrameData(ref string strInData)호출시 인자로 전달
            strInData = strTemp.Remove(iPosSTX, (iPosETX - iPosSTX) + 1);

            // @와;가 들어있는 완전한 프레임을 반환
            return strRet;
        }

        // Frame Format : STX(1) + LENGTH(5) + CRC(8) + DATA(N) + ETX(1)
        public VERIFYCODE Verification(string strFrameData)
        {
            // 데이터길이
            int iLength_Read;
            // 데이터길이 검증
            int iLength_Cal;
            // 수신프레임에서 추출된 CRC저장
            string strCRC_Read;
            // strCRC_Read(16진수) --> int형으로 변환
            int iCRC_Read;
            // CRC값을 계산하고 int형으로 변환
            int iCRC_Cal;
            string strData;
            byte[] chData;

            try
            {
                // 수신데이터에서 데이터의 길이 추출
                iLength_Read = Convert.ToInt32(strFrameData.Substring(1, 5));
                // 수신데이터의 데이터의 길이 계산
                iLength_Cal = strFrameData.Length - 15;

                if (iLength_Read != iLength_Cal)
                {
                    // 길이 오류
                    return VERIFYCODE.LENGTH_ERROR;
                }

                // 수신된 데이터에서 실제데이터만 추출
                strData = strFrameData.Substring(14, strFrameData.Length - 15);

                // CRC추출
                strCRC_Read = strFrameData.Substring(6, 8);

                // 16진수를 int형으로 변환하여 iCRC_Read에 저장
                iCRC_Read = Convert.ToInt32(strCRC_Read, 16);

                // String형을 Unicode타입의 Byte[]로 변환
                chData = Encoding.Unicode.GetBytes(strData);

                // CRC값을 계산하고 Int형으로 변환하여 iCRC_Cal에 저장
                iCRC_Cal = (int)SmartX.SmartTCPMultiServer.GetCRC32Gen(chData);
            }
            catch (Exception)
            {
                return VERIFYCODE.ERROR;
            }

            // 수신데이터의 CRC32값과 계산하여 만든 CRC32값이 일치 안한다면
            if (iCRC_Read != iCRC_Cal)
            {
                return VERIFYCODE.CRC_ERROR;
            }

            return VERIFYCODE.SUCESS;
        }

        private void smartTCPMultiServer1_OnClientAdd(SmartX.SmartTCPMultiServer.CLIENTSINFOS addClientInfo)
        {
            m_strClientIPAddress = addClientInfo.strIPAddress;
        }
    }
}